/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
*     This class is the container class of the CSysApShutdownTone.
*     Is used to play shutdown tone.
*
*/


// INCLUDE FILES
#include <Profile.hrh>
#include <centralrepository.h>
#include <startupdomaincrkeys.h>
#include <ProfileEngineSDKCRKeys.h>
#include "SysApShutdownTone.h"
#include "SysApShutdownAnimation.h"
#include "SysAp.hrh"
#include "AudioPreference.h"

//=============================== MEMBER FUNCTIONS ============================
// ---------------------------------------------------------
// Constructor
// ---------------------------------------------------------
//
CSysApShutdownTone::CSysApShutdownTone( CSysApShutdownAnimation* aCallback ) :
    iCallback( aCallback ),
    iTone( NULL ),
    iAudioReady( EFalse ),
    iPlaying( EFalse )
    {
    }

// ---------------------------------------------------------
// Destructor 
// ---------------------------------------------------------
//
CSysApShutdownTone::~CSysApShutdownTone()
{
    

    if (iTone)
        {
        if (iPlaying)
            {
            
            iTone->Stop();
            }
        delete iTone;
        iTone = NULL;
        }
}

// ----------------------------------------------------
// CSysApShutdownTone::NewL()
// ----------------------------------------------------
CSysApShutdownTone* CSysApShutdownTone::NewL(TDesC& aShutdownTonePath, CSysApShutdownAnimation* aCallback)
{
    CSysApShutdownTone* self = new (ELeave) CSysApShutdownTone(aCallback);
    
    CleanupStack::PushL( self );
    self->ConstructL(aShutdownTonePath);
    CleanupStack::Pop(); // self

    return self;
}

// ----------------------------------------------------
// CSysApShutdownTone::ConstructL()
// ----------------------------------------------------
void CSysApShutdownTone::ConstructL(TDesC& aShutdownTonePath)
{
    
    // Check tone volume
    iVolume = GetRingingToneVolumeL();
    TInt err( KErrNone );

    if (iVolume)
        {
        TPath tonePath;
        

        RFs fs;
        err = fs.Connect();
        TFindFile findExe(fs);
        err = findExe.FindByPath( aShutdownTonePath, NULL );
        fs.Close();
        if (err != KErrNone)
            {
            
            User::Leave(err);
            }
        else
            {
            
            iTone = CMdaAudioPlayerUtility::NewFilePlayerL(
                        aShutdownTonePath, 
                        *this, KAudioPriorityPhonePower, 
                        TMdaPriorityPreference( KAudioPrefDefaultTone));
            }
        }
    else // Phone is in silent mode, tone can be set to be ready instantly
        {
        
        
        if ( iCallback )
            {
            iCallback->ToneReady();
            }
        }            
}
// ---------------------------------------------------------
// void CSysApShutdownTone::Play()
// ---------------------------------------------------------
//

TInt CSysApShutdownTone::Play()
{
    
    if (iAudioReady && iVolume && iTone)
        {
        
        iVolume = Max( 0, Min( iVolume, 10 ) );
        iTone->SetVolume(iVolume);
        iTone->Play();
        iPlaying = ETrue;
        return KErrNone;
        }
    else
        {
        
        
        
        
        return KErrNotReady;
        }
}

// ---------------------------------------------------------
// void CSysApShutdownTone::Stop()
// ---------------------------------------------------------
//
void CSysApShutdownTone::Stop()
{
    
    if (iTone)
        {
        iPlaying=EFalse;
        iTone->Stop();
        }
    
    // Set null to callback as notifying tone readiness is no longer needed and causes incorrect
    // functionality.
    iCallback = NULL;
}

// ---------------------------------------------------------
// void CSysApShutdownTone::MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& aDuration)
// ---------------------------------------------------------
//
void CSysApShutdownTone::MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& /*aDuration*/)
{
    
    if (aError == KErrNone)
        {
        
        iAudioReady = ETrue;
        }
    else
        {
        
        }
    
    if ( iCallback )
        {
        iCallback->ToneReady();
        }
}

// ---------------------------------------------------------
// void CSysApShutdownTone::MapcPlayComplete(TInt /*aError*/)
// ---------------------------------------------------------
//
void CSysApShutdownTone::MapcPlayComplete(TInt /*aError*/)
{
    
    iPlaying=EFalse;
}

// ---------------------------------------------------------
// TBool CSysApShutdownTone::Playing()
// ---------------------------------------------------------
//
TBool CSysApShutdownTone::Playing()
{
    
    return iPlaying;
}

// ----------------------------------------------------------
// CSysApShutdownTone::GetRingingToneVolumeL
// Shutdown tone volume is always 4 but when ringing type is 
// silent or ringing volume is 0 or 1 startup tone is silent.
// ----------------------------------------------------------
//
TInt CSysApShutdownTone::GetRingingToneVolumeL()
    {
    

    TInt retval(0);
    TInt ringingType(EProfileRingingTypeSilent);
    TInt ringingVol(0);
    
    CRepository* repository = NULL;

    TRAPD( err, repository = CRepository::NewL( KCRUidProfileEngine ) );
    if ( err != KErrNone )
        {
        return 0;    
        }
    
    if ( repository )
        {
        CleanupStack::PushL(repository);        
        User::LeaveIfError( repository->Get( KProEngActiveRingingVolume, ringingVol ));
        User::LeaveIfError( repository->Get( KProEngActiveRingingType, ringingType ));
        CleanupStack::PopAndDestroy(repository);
        }

    
    

    if ((ringingType != EProfileRingingTypeSilent) && 
        (ringingVol != 0))
        {
        TInt defaultRingingVol;
        CRepository* repository = NULL;

        TRAPD( err, repository = CRepository::NewL( KCRUidStartupConf ) );
        if ( err != KErrNone )
            {
            return 0;    
            }

        if ( repository )
            {
            CleanupStack::PushL(repository);
            User::LeaveIfError( repository->Get( KStartupToneVolume, defaultRingingVol ));
            CleanupStack::PopAndDestroy(repository);
            }

        ringingVol = defaultRingingVol;
        retval =  ringingVol;
        }

    
    return retval;
    }

// End of File
